###########################################################################################################################
#                                        Identification of water bridges                                                  #
###########################################################################################################################
# Importing the required modules
import os, re, itertools

# A used defined function to remove ambiguity arising due to hydrogen bond donor as well as acceptor nature of 
# some atoms as HBPLUS prints the same interaction twice by exchanging the role (hydrogen bond donor as well as 
# acceptor) of interacting atoms.
def uq_interactions(set1):
    extra = []
    for h in range(len(set1)):
        for i in range(h+1,len(set1)):
            if (set1[h][:27] == set1[i][:27]):
                extra.append(set1[h])
    for h in extra:
        if h in set1:
            set1.remove(h)

# List of nucleotides and amino acids.
nucleotides = ['-  A', '-  C', '-  G', '-  U']
amino_acid = ['-GLY', '-ALA', '-SER', '-THR', '-CYS', '-VAL', '-LEU', '-ILE', '-MET', '-PRO', '-PHE', '-TYR',
              '-TRP', '-ASP', '-GLU', '-ASN', '-GLN', '-HIS', '-LYS', '-ARG']

# Look for output files of HBPLUS (.hb2) present in the current directory.
files_hb2 = []
path = os.getcwd()
directory = os.listdir(path)  
for fis in directory: 
    if fis.find('.hb2') >= 0:
        files_hb2.append(fis)
files_hb2.sort()

# Analyzing each .hb2 file recursively to identify water bridges.
for f in files_hb2:
    file = open(f, 'r') # Read .hb2 files
    data = file.readlines()
    hb_water = [] # Created a list of hydrogen bonds of amino acids and nucleotides with water molecules.
    new_data = [] # Created a list of all direct hydrogen bonds between amino acids and nucleotides.
    
    for l in range(len(data)):
        # Rearranging the format of writing hydrogen bonds of water molecules with amino acids or nucleotides; 
        # information of water molecules will be written first followed by amino acid or nucleotide 
        # (A2601-HOH O   A0018-ASN N   3.28).  
        if data[l].count('-HOH') == 1:
            p1 = data[l][:14]
            p2 = data[l][14:28] 
            p3 = data[l][28:33]
            if 'HOH' in p1 and p2[5:9] in nucleotides or 'HOH' in p1 and p2[5:9] in amino_acid:
                r = p1 + p2 + p3
                hb_water.append(r.strip())
            if 'HOH' in p2 and p1[5:9] in nucleotides or 'HOH' in p2 and p1[5:9] in amino_acid: 
                r = p2 + p1 + p3
                hb_water.append(r.strip())
            
        # Rearranging the format of writing direct hydrogen bonds between amino acids and nucleotides; 
        # information of nucleotide will be written first followed by amino acid (C0105-  G N7  A0043-ASN N   2.90). 
        if data[l].count('-HOH') == 0:
            p1 = data[l][:14]
            p2 = data[l][14:28] 
            p3 = data[l][28:33]
            if 'HOH' not in p1 and 'HOH' not in p2:
                if p1[5:9] in nucleotides and p2[5:9] in amino_acid:
                    ra = p1 + p2 + p3
                    if ra not in new_data: new_data.append(ra.strip())
                if p2[5:9] in nucleotides and p1[5:9] in amino_acid:
                    ra = p2 + p1 + p3
                    if ra not in new_data: new_data.append(ra.strip())
                        
    # Removing redundant hydrogen bonds arising due to the dual nature of some atoms.
    uq_interactions(new_data) # This will remove repetitions from the list 'new_data'.     
    uq_interactions(hb_water)  # This will remove repetitions from the list 'hb_water'.
    
    wtr = [] # Created a list of unique water molecules using their identity as per PDB record.
    for i in hb_water:
        if i[:13].strip() not in wtr:
            wtr.append(i[:13].strip())
    
    # Sorting all hydrogen bonds of water molecules and placing all hydrogen bonds of particular water adjacent to each other
    # to each other.
    wtr.sort()
    sorted_hbs = []
    for w1 in wtr:
        for w2 in hb_water:
            if w1 in w2:
                sorted_hbs.append(w2)
                
    # Grouping all hydrogen bonds of a particular water molecule, resulting in a water-mediated motif of amino acids 
    # and nucleotides.
    all_inter_water = [] # a list consisting of a list of all hydrogen bonds of water with amino acids and nucleotides. 
    iterator = itertools.groupby(sorted_hbs, lambda string: string.split('H O   ')[0])
    
    # Appending the group by converting it into a list;
    # here 'element' is information of water molecule, on the basis of which all hydrogen bonds of a particular 
    # water molecules are grouped and 'group' is all interaction of that water molecule. 
    for element, group in iterator:
        all_inter_water.append(list(group))
    all_inter_water_joined = []
    for i in all_inter_water:
        all_inter_water_joined.append(" --- ".join(i))
        
    # Improving the output of the code.
    result_1 = [] # list of grouped hydrogen bonds of each water molecule. 
    for i in all_inter_water_joined:
        x = i.split(i[:14])
        xx = ''.join(x)
        xxx = i[:14].strip() + ' --- ' + xx
        result_1.append(xxx)
    
    # Filtering out false water bridges (each water should form at least one hydrogen bond with amino acid and nucleotide).
    r1 = re.compile(".*-  [A,C,G,U] .*")
    newlist1 = list(filter(r1.match, result_1))
    r2 = re.compile(".*(GLY|ALA|SER|THR|CYS|VAL|LEU|ILE|MET|PRO|PHE|TYR|TRP|ASP|GLU|ASN|GLN|HIS|LYS|ARG).*")
    wtr_brg = list(filter(r2.match, newlist1))
    
    # Adding more information to the water bridge; whether cyclic or acyclic.
    # For this, hydrogen bonds of amino acids and nucleotides involved in water bridge formation will be searched
    # in previously created list 'new_data'.
    my_dict = {}
    key = []
    val = []
    for i in wtr_brg:
        NB = [] # List of nucleotides forming hydrogen bonds with water.
        AA = [] # List amino acids forming hydrogen bonds with water.
        for a in range(16,len(i),23):
            if i[a:a+9] not in NB and i[a:a+9][5:] in nucleotides: 
                NB.append(i[a:a+9])
            if i[a:a+9] not in AA and i[a:a+9][5:] in amino_acid: 
                AA.append(i[a:a+9])   
        for n in range(len(NB)):
            for m in range(len(AA)):
                for l in (new_data): 
                    if NB[n] in l and AA[m] in l:
                        key.append(i)
                        val.append(l[:33])
                    
        if i not in key:
            key.append(i)
            val.append('NIL')
            
    for i in range(len(key)):
        for j in range(len(val)):
            if i == j: 
                my_dict.setdefault(key[i], []).append(val[j])
                
    nucleobase_atoms = ['N9', 'N6', 'N2', 'N7', 'O4', 'O6', 'N1', 'N4', 'N3', 'O2']
    
    # Generate output files, here we generated three output files 
    print(('********************  ' + f + '  ********************'), file=open("waterbridges.txt", "a"))
    #print(('********************  ' + f + '  ********************'), file=open("waterbridges_Base.txt", "a"))
    #print(('********************  ' + f + '  ********************'), file=open("waterbridges_RiboPhos.txt", "a"))
    
    
    #for a, b in my_dict.items():
    #
    #    z = 0
    #    for xa in range(26, len(a), 23):
    #        if (a[xa:xa+3].strip()) in nucleobase_atoms:
    #            z += 1
    #    if z >= 1:
    #        print(a,b, file=open("waterbridges_Base.txt", "a"))
    #    if z == 0:
    #        print(a,b, file=open("waterbridges_RiboPhos.txt", "a"))  
    #print('........................................................................ \n', file=open("waterbridges_Base.txt", "a"))
    #print('........................................................................ \n', file=open("waterbridges_RiboPhos.txt", "a"))
   
    
    for a, b in my_dict.items():
        print(a,b, file=open("waterbridges.txt", "a"))

    print('........................................................................ \n', file=open("waterbridges.txt", "a"))



import subprocess
import sys
import os

def run_script(script_name):
    script_dir = os.path.dirname(os.path.realpath(__file__))
    script_path = os.path.join(script_dir, script_name)
    try:
        result = subprocess.run(['python', script_path], check=True)
    except subprocess.CalledProcessError as e:
        print(f"Error occurred while running {script_name}: {e}")

def main():
    if len(sys.argv) != 2:
        print("""
How to run: waterbridge -1 | -2 | -3

-1: Identify water bridges in RNA–protein complexes.
    - For this, ensure that the output files from HBPLUS are placed in the current directory.

-2: Analyze the output of option -1 and identify the topology of the water bridges, such as A1-w-N1, A1-w-N2, and others.

-3: Provide nomenclature for identified topologies.
""")
        sys.exit(1)

    arg = sys.argv[1]
    
    if arg == '-1':
        run_script('1_Identification of water bridges.py')
    elif arg == '-2':
        run_script('2_Topological classification of water bridges.py')
    elif arg == '-3':
        run_script('3_Nomenclature of waterbridges.py')
    else:
        print("Please enter a valid argument; -1, -2, or -3.")
        sys.exit(1)

if __name__ == "__main__":
    main()



###########################################################################################################################
#                                      Topological classification of water bridges                                        #
###########################################################################################################################
# Importing the required modules.
import os

# List of nucleotides and amino acids.
NT = ['-  A', '-  U', '-  G', '-  C']
AA = ['-GLY', '-ALA', '-SER', '-THR', '-CYS', '-VAL', '-LEU', '-ILE', '-MET', '-PRO', '-PHE', '-TYR', '-TRP', '-ASP', '-GLU', '-ASN', '-GLN', '-HIS', '-LYS', '-ARG']


# User-defined function to extract PDB record and identity of interacting amino acids and nucleotides. 
# List 'aanb_info' contains PDB record (chain number, residue number) of interacting amino acids and 
# nucleotides. For example: 'Q0030-THR'. 
# List 'aa_nt' contains the identity of interacting amino acids and nucleotides. For example: 'THR'. 
def aa_nt_name(splitted_list_name):
    for i in range(len(splitted_list_name)):
        if i+1 < len(splitted_list_name):
            if splitted_list_name[i+1][:10] not in aanb_info:
                aanb_info.append(splitted_list_name[i+1][:10])
                                                                
    for j in aanb_info:                                                    
        aa_nt.append(j[6:])

# Read any of the three output files of previously identified water bridges.
file_waterbridges = 'waterbridges.txt'
file = open(file_waterbridges, 'r')
i = -1
start_line_plus_first_line = []
end_line = []
my_dict = {"pdb":[],"val":[]}
pdb = []
data = file.readlines()
for lines in data: 
    i += 1
    if lines.find('.hb2') >= 0:
        start_line_plus_first_line.append(i)
    elif lines.find('.....................................') >= 0:
        end_line.append(i)

for n in range(len(start_line_plus_first_line)):
    for m in range(len(end_line)):
        if n == m:
            my_dict["pdb"].append(data[start_line_plus_first_line[n]])
            my_dict["val"].append(data[start_line_plus_first_line[n]+1:end_line[m]])

for l in range(len(my_dict['val'])):
    for nm in my_dict['val'][l]:
        # List contains PDB record (chain number, residue number) of interacting amino acids and
        # nucleotides. For example - Q0030-THR. 
        aanb_info = []
        # List 'aa_nt' contains identity of interacting amino acids and nucleotides. For example: 'THR'. 
        aa_nt = []
        x1 = nm.split('[')[0]  # x1 contains the upper topology only
        x2 = x1.split('---') # x2 is direct hydrogen bonds b/w amino acid and nucleotide.
        aa_nt_name(x2) # Extracting PDB record and identity of interacting amino acids and nucleotides.
        
        # Counting the number of amino acids and nucleotides involved in water bridge formation.
        aa =  len([i for i in aa_nt if i in AA ])
        nt =  len([i for i in aa_nt if i in NT ])
        
        # Saving water bridges as per their topology.
        filename = 'A' + str(aa) + '-w-N' + str(nt) + '.txt' 
        path = os.getcwd()
        directory = os.listdir(path)  # specify folder path.
        if filename in directory: 
            out_files = open(filename, 'r')
            if my_dict['pdb'][l] not in out_files: 
                print(my_dict['pdb'][l], file=open(filename, "a"))
        else:
            print(my_dict['pdb'][l], file=open(filename, "a"))
        print(nm, file=open(filename, "a"))
        
        



from setuptools import setup, find_packages


setup(
    name='waterbridge',  # Package name
    version='0.2.0',      # Version
    description='A tool for identifying water bridges and their topology in RNA–protein complexes.',
    long_description=open('README.md').read(),  # Read the README file
    long_description_content_type='text/markdown',
    author='Raman Jangra',
    author_email='raman.compchem@gmail.com',
    url='https://github.com/RamanCompChem/waterbridges',  # Link to the project repository
    packages=find_packages(),  # Automatically find and include all packages
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',   # Specify minimum Python version
    entry_points={
        'console_scripts': [
            'waterbridge=waterbridge.waterbridge:main',  # Command line entry point
        ],
    },
)



############################################################################################################################
#                                       Nomenclature of waterbridges                                                       #
############################################################################################################################
import math
import os

to_update = {':A': ':rA', ':U': ':rU',':G': ':rG',':C': ':rC',}

def group_list_prefix(old_list):
    grouped = {}
    for item in old_list:
        prefix = item[:9]
        if prefix not in grouped:
            grouped[prefix] = []
        grouped[prefix].append(item)
    new_list = list(grouped.values())
    return new_list

# User-defined function to convert PDB record of a water molecule from HBPLUS format (B0026-HOH) 
# to original PDB format (HOH B 26).  
def convert_water_to_pdb(W_id):
    W_id_a = W_id[0]
    W_id_b = W_id[1:5]
    W_id = W_id_a + str(int(W_id_b))
    
    W_id = list(W_id)
    while len(W_id) <5:
        W_id.insert(1, ' ')
    W_id = ''.join(W_id)
    w_pdb = 'O   HOH ' + W_id
    return w_pdb

# User-defined function to convert PDB record of nucleotide to HBPLUS format to original PDB format.  
def convert_nt_to_pdb(idd):
    id_a = idd[0]
    id_b = idd[1:5]
    id_c = idd[8]
    id_atom = idd[10:13]
    chainresi = id_a + str(int(id_b))
    chainresi = list(chainresi)
    while len(chainresi) <5:
        chainresi.insert(1, ' ')
    chainresi.insert(0, ' ' )
    chainresi.insert(0, id_c)
    chainresi.insert(0, id_atom)
    while len(chainresi) <11:
        chainresi.insert(1, ' ')
    nt_pdb = ''.join(chainresi)
    return nt_pdb

# User-defined function to decide interacting edge of nucleotide.
def decision(nt1, nt2, hoh):
    global nt1_xyz, nt2_xyz, hoh_xyz 
    for cords in cord_data:
        if hoh in cords and 'HETATM' in cords: 
            hoh_xyz = [float(cords[31:38].strip()), float(cords[39:46].strip()), float(cords[47:55].strip())]
        if nt1 in cords and 'ATOM' in cords: 
            nt1_xyz = [float(cords[31:38].strip()), float(cords[39:46].strip()), float(cords[47:55].strip())]
        if nt2 in cords and 'ATOM' in cords: 
            nt2_xyz = [float(cords[31:38].strip()), float(cords[39:46].strip()), float(cords[47:55].strip())]
    if (math.dist(hoh_xyz, nt1_xyz)) < (math.dist(hoh_xyz, nt2_xyz)): near_atom = nt1[0:3].strip()
    else: near_atom = nt2[0:3].strip()
    return near_atom
    
# User-defined function to sort ribonucleotides in nomenclature:
def sort_nucleos(coded_ribonucleotides_list):
    WC_list = [nu for nu in coded_ribonucleotides_list if '(WC)' in nu] 
    HG_list = [nu for nu in coded_ribonucleotides_list if '(HG)' in nu]
    SG_list = [nu for nu in coded_ribonucleotides_list if '(SG)' in nu]
    PH_list = [nu for nu in coded_ribonucleotides_list if '(Ph)' in nu]
    RB_list = [nu for nu in coded_ribonucleotides_list if '(Rb)' in nu]
    
    WC_list.sort() 
    HG_list.sort()
    SG_list.sort()
    PH_list.sort()
    RB_list.sort()
    
    final_parts = []
    
    # Add parts in the specified order
    if WC_list: final_parts.append('|'.join(WC_list))
    if HG_list: final_parts.append('|'.join(HG_list))
    if SG_list: final_parts.append('|'.join(SG_list))
    if PH_list: final_parts.append('|'.join(PH_list))
    if RB_list: final_parts.append('|'.join(RB_list))
    
    final_string = '|'.join(final_parts)
    return final_string

# User-defined function to sort amino acids in nomenclature:
def sort_amino_acids(coded_amino_acids_list):
    # part 1: categorize amino acids based on their suffixes
    main_chain_list = []
    main_chain_side_list = []
    side_chain_list = []
    
    for acid in coded_amino_acids_list:
        if '(m)' in acid: main_chain_list.append(acid)
        elif '(ms)' in acid: main_chain_side_list.append(acid)
        elif '(s)' in acid: side_chain_list.append(acid)
            
    # sort each category in reverse order as amino acids will be written on left side of :w:
    main_chain_list.sort(reverse=True)
    main_chain_side_list.sort(reverse=True)
    side_chain_list.sort(reverse=True)
    
    # exclude empty lists
    final_parts = []
    if side_chain_list: final_parts.append('|'.join(side_chain_list))
    if main_chain_side_list: final_parts.append('|'.join(main_chain_side_list))
    if main_chain_list: final_parts.append('|'.join(main_chain_list))
    
    final_string = '|'.join(final_parts)
    return final_string

# List of nucleotides and amino acids.
NT = ['-  A', '-  U', '-  G', '-  C']
AA = ['-GLY', '-ALA', '-SER', '-THR', '-CYS', '-VAL', '-LEU', '-ILE', '-MET', '-PRO', '-PHE', '-TYR', '-TRP', '-ASP', '-GLU', '-ASN', '-GLN', '-HIS', '-LYS', '-ARG']

# Combination of atoms to decide interacting portion of amino acid
MC = ['O  ', 'N  ', 'OXT', ['N  ', 'O  '], ['N  ', 'OXT'], ['O  ', 'OXT'], ['N  ', 'O  ', 'OXT']] # Main chain
mc_atoms = ['O', 'N', 'OXT']
# Combination of atoms to decide interacting portion of amino acid
WC_edge = [['N1', 'N6'],  ['N1', 'N2'], ['N1', 'O6'],  ['N3', 'N4'], ['N3', 'O2'], ['N3', 'O4']] # Watson-Crick edge
HG_edge = [['N6', 'N7'],  ['N7', 'O6'], ['N7', 'O5\'']] # Hoogsteen edge
SG_edge = [['N3', 'N9'], ['N3', 'O2\''], ['N1', 'O2'], ['N2', 'N3'], ['O2', 'O2\''], ['O2\'', 'O3\''], ['N3', 'O4\''], ['N3', "O2\'", "O3\'"]] # Sugar edge
rib1 = [['O4\''], ['O3\'']]
rib2 = [['O3\'', 'O4\''], ['O4\'', 'O5\''],['O2\'', 'O3\'']]
phos1 = [['OP1'], ['OP2'], ['O5\'']]
phos2 = [['OP1', 'OP2'], ['O5\'', 'OP1'], ['O5\'', 'OP2']]

def nucleotide_three_atoms(ntt):
    nt_atoms = []
    for nt in ntt:
        nt_a = nt[-3:].strip()  # interacting atoms
        nt_resi = str(int(nt[1:5])) # residue number
        nt_nt = 'r' + nt[8] # nucleotide
        nt_F = nt_nt + nt_resi
        nt_atoms.append(nt_a)
    nt_atoms.sort()
    if nt_atoms in SG_edge: z_n1 = '(SG)'
    else: z_n1 = 'ERROR'
    final = nt_F + z_n1
    return final

def nucleotide_two_atoms(ntt):
    nt_atoms = []
    for nt in ntt:
        nt_a = nt[-3:].strip()  # interacting atoms
        nt_resi = str(int(nt[1:5])) # residue number
        nt_nt = 'r' + nt[8] # nucleotide
        nt_F = nt_nt + nt_resi
        nt_atoms.append(nt_a)
    nt_atoms.sort()
    if nt_atoms in WC_edge: z_n1 = '(WC)'
    elif nt_atoms in HG_edge: z_n1 = '(HG)'
    elif nt_atoms in SG_edge: z_n1 = '(SG)'
    elif nt_atoms in phos2: z_n1 = '(Ph)'
    elif nt_atoms in rib2: z_n1 = '(Rb)'
    else: z_n1 = 'ERROR'
    final = nt_F + z_n1
    return final

def nucleotide_one_atom(ntt):
    nt = ntt[0] + ' ' # nnt is a list of single entry, so for using that entry as a string 
    nt_a = nt[-3:]  # interacting atoms
    nt_resi = str(int(nt[1:5])) # residue number
    nt_nt = 'r' + nt[8] # nucleotide
    nt_F = nt_nt + nt_resi
    w_pdb =  convert_water_to_pdb(x)    
    if '-  A' in nt:
        if ' N6 ' in nt:
            new_n_h = nt.replace('N6', 'C5')
            new_n_w = nt.replace('N6', 'N1')
            nt_pdb_h = convert_nt_to_pdb(new_n_h)
            nt_pdb_w = convert_nt_to_pdb(new_n_w)
            nearest_atom = decision(nt_pdb_h, nt_pdb_w, w_pdb)
            if nearest_atom == 'C5': z_n1 = '(HG)'
            else: z_n1 = '(WC)'
        elif ' N7 ' in nt: z_n1 = '(HG)'
        elif ' N1 ' in nt: z_n1 = '(WC)'
        elif ' N3 ' in nt: z_n1 = '(SG)'
        elif ' N9 ' in nt: z_n1 = '(SG)'
        elif ' O2\'' in nt: z_n1 = '(SG)'
        elif ' O4\'' in nt: z_n1 = '(Rb)'
        elif ' O3\'' in nt: z_n1 = '(Rb)'
        elif ' OP1' in nt: z_n1 = '(Ph)'
        elif ' OP2' in nt: z_n1 = '(Ph)'
        elif ' O5\'' in nt: z_n1 = '(Ph)'
        else: z_n1 = 'ERR'

    elif '-  G' in nt:
        if ' O6 ' in nt:
            new_n_h = nt.replace('O6', 'C5')
            new_n_w = nt.replace('O6', 'N1')
            nt_pdb_h = convert_nt_to_pdb(new_n_h)
            nt_pdb_w = convert_nt_to_pdb(new_n_w)
            nearest_atom = decision(nt_pdb_h, nt_pdb_w, w_pdb) #nt_pdb_h will decide edge
            if nearest_atom == 'C5': z_n1 = '(HG)'
            else: z_n1 = '(WC)'
        elif ' N2 ' in nt:
            new_n_s = nt.replace('N2', 'N3')
            new_n_w = nt.replace('N2', 'N1')
            nt_pdb_s = convert_nt_to_pdb(new_n_s)
            nt_pdb_w = convert_nt_to_pdb(new_n_w)
            nearest_atom = decision(nt_pdb_s, nt_pdb_w, w_pdb) #nt_pdb_s will decide edge
            if nearest_atom == 'N3': z_n1 = '(SG)'
            else: z_n1 = '(WC)'
        elif ' N7 ' in nt: z_n1 = '(HG)'
        elif ' N1 ' in nt: z_n1 = '(WC)'
        elif ' N3 ' in nt: z_n1 = '(SG)'
        elif ' N9 ' in nt: z_n1 = '(SG)'
        elif ' O2\'' in nt: z_n1 = '(SG)'
        elif ' O4\'' in nt: z_n1 = '(Rb)'
        elif ' O3\'' in nt: z_n1 = '(Rb)'
        elif ' OP1' in nt: z_n1 = '(Ph)'
        elif ' OP2' in nt: z_n1 = '(Ph)'
        elif ' O5\'' in nt: z_n1 = '(Ph)'
        else: z_n1 = 'ERR'

    elif '-  U' in nt:
        if ' O4 ' in nt:
            new_n_h = nt.replace('O4', 'C5')
            new_n_w = nt.replace('O4', 'N3')
            nt_pdb_h = convert_nt_to_pdb(new_n_h)
            nt_pdb_w = convert_nt_to_pdb(new_n_w)
            nearest_atom = decision(nt_pdb_h, nt_pdb_w, w_pdb) #nt_pdb_s will decide edge
            if nearest_atom == 'C5': z_n1 = '(HG)'
            else: z_n1 = '(WC)' 
        elif ' O2 ' in nt:
            new_n_s = nt.replace('O2', 'N1')
            new_n_w = nt.replace('O2', 'N3')
            nt_pdb_s = convert_nt_to_pdb(new_n_s)
            nt_pdb_w = convert_nt_to_pdb(new_n_w)
            nearest_atom = decision(nt_pdb_s, nt_pdb_w, w_pdb) #nt_pdb_s will decide edge
            if nearest_atom == 'N1': z_n1 = '(SG)'
            else: z_n1 = '(WC)' 
        elif ' N1 ' in nt: z_n1 = '(SG)'
        elif ' N3 ' in nt: z_n1 = '(WC)'
        elif ' O2\'' in nt: z_n1 = '(SG)'
        elif ' O4\'' in nt: z_n1 = '(Rb)'
        elif ' O3\'' in nt: z_n1 = '(Rb)'
        elif ' OP1' in nt: z_n1 = '(Ph)'
        elif ' OP2' in nt: z_n1 = '(Ph)'
        elif ' O5\'' in nt: z_n1 = '(Ph)'
        else: z_n1 = 'ERR'
            
    elif '-  C' in nt:
        if ' O2 ' in nt:
            new_n_s = nt.replace('O2', 'N1')
            new_n_w = nt.replace('O2', 'N3')
            nt_pdb_s = convert_nt_to_pdb(new_n_s)
            nt_pdb_w = convert_nt_to_pdb(new_n_w)
            nearest_atom = decision(nt_pdb_s, nt_pdb_w, w_pdb) #nt_pdb_s will decide edge
            if nearest_atom == 'N1': z_n1 = '(SG)'
            else: z_n1 = '(WC)' 
        elif ' N4 ' in nt:
            new_n_h = nt.replace('N4', 'C5')
            new_n_w = nt.replace('N4', 'N3')
            nt_pdb_h = convert_nt_to_pdb(new_n_h)
            nt_pdb_w = convert_nt_to_pdb(new_n_w)
            nearest_atom = decision(nt_pdb_h, nt_pdb_w, w_pdb) #nt_pdb_s will decide edge
            if nearest_atom == 'C5': z_n1 = '(HG)'
            else: z_n1 = '(WC)' 
        elif ' O2\' ' in nt: z_n1 = '(SG)'
        elif ' N1 ' in nt: z_n1 = '(SG)'
        elif ' N3 ' in nt: z_n1 = '(WC)'
        elif ' O4\'' in nt: z_n1 = '(Rb)'
        elif ' O3\'' in nt: z_n1 = '(Rb)'
        elif ' OP1' in nt: z_n1 = '(Ph)'
        elif ' OP2' in nt: z_n1 = '(Ph)'
        elif ' O5\'' in nt: z_n1 = '(Ph)'
        else: z_n1 = 'ERR'
    final = nt_F + z_n1
    return final
        


files_from_previous = []
path = os.getcwd()
directory = os.listdir(path)  
for fis in directory: 
    if '.txt' in fis and '-w-' in fis and 'waterbridges' not in fis :
        files_from_previous.append(fis)
files_from_previous.sort()

# Read previously generated file containing water bridges.
for previousfile in files_from_previous:
    file = open(previousfile, 'r')
    outfile = 'named_'+ previousfile[:2] + 'w' + previousfile[5:7] + '_waterbridges.txt'
    data = file.readlines()
    for lines in data:
        print(lines)
        if 'pdb' in lines: 
            #print(lines)
            # Reading PDB files to extract coordinates of atoms.
            cord_file = open(lines.split('.')[0].split(' ')[2] + '.ent')
            cord_data = cord_file.readlines()
            print(lines.strip(), file=open(outfile, "a"))
        if 'pdb' not in lines and 'HOH' in lines:
            AA_info = [] # List of amino acids.
            NT_info = [] # List of nucleotides.
            # Determine whether the water bridge is cyclic or acyclic.
            if "NIL" in lines: zc = ''
            else: zc = 'cyc-'
            # Extracting amino acids and nucleotides.
            x = lines.split('[')[0]
            for i in range(16, len(x), 23):
                y = lines[i:i+14].strip()
                y1 = y[5:9]
                if y1 in NT: 
                    NT_info.append(y) 
                elif y1 in AA: 
                    AA_info.append(y)
            AA_info = group_list_prefix(AA_info)
            NT_info = group_list_prefix(NT_info)
            #print(AA_info, NT_info)
            
            code_nt_ALL = [] # contains all coded nucleotides
            for ntt in NT_info:
                # Case 1: When 3 atoms of a nucleotide are forming hydrogen bonds with water.
                if len(ntt) == 3:
                    z_n = nucleotide_three_atoms(ntt)
                    #print(z_n)
                # Case 2: When 2 atoms of a nucleotide are forming hydrogen bonds with water.
                elif len(ntt) == 2:
                    z_n = nucleotide_two_atoms(ntt)
                    #print(z_n)
                # Case 3: When only 1 atom of nucleotide is forming a hydrogen bond with water.
                elif len(ntt) == 1:
                    z_n = nucleotide_one_atom(ntt)
                    #print(z_n)
                code_nt_ALL.append(z_n)
        
            #print(code_nt_ALL)
            right_nt = sort_nucleos(code_nt_ALL)
            #print(right_nt)
            
            # Deciding interacting portion of amino acid.
            code_aa_ALL = [] # contains all coded nucleotides
            for aat in AA_info:
                aa_id_ms = []
                #print(aat)
                for aa in aat:
                    aa_resi = str(int(aa[1:5])) # residue number
                    aa_aa = aa [6:9] # amino acid
                    aa_atom = aa[10:].strip()  # interacting amino acid atom
                    if aa_atom in mc_atoms:
                        z_a = 'm'
                    else: z_a = 's'
                    if z_a not in aa_id_ms: aa_id_ms.append(z_a) 
                aa_id_ms.sort()
                joined_mc_sc = ''.join(aa_id_ms)
                code_aa = aa_aa + aa_resi + '(' + joined_mc_sc + ')'
                code_aa_ALL.append(code_aa)
            #print(code_aa_ALL)
            left_aa = sort_amino_acids(code_aa_ALL)
            #print(left_aa)
            
            name =  zc + left_aa + ':w:' + right_nt
            interaction = lines.strip() + ' —> ' + name
            print(interaction)
            print(interaction.strip(), file=open(outfile, "a"))
    file.close()


                


            
        
                
                 
            
            
            
            
            

